#include <assert.h>

#include "path.h"
#include "map.h"
#include "robot.h"

void map_set_tiletype(map_t *map, size_t x, size_t y, uint8_t value)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].type = value;
}

map_t *map_read(sockbuf_t *sock, int width, int height)
{
	map_t *map;
	int x, y;

	map = (map_t *) malloc(sizeof(map_t));
	map->width = width;
	map->height = height;
	map->data = (tile_t *) malloc(width * height * sizeof(tile_t));
	for (y = 1; y <= height; y++) {
		for (x = 1; x <= width; x++) {
			unsigned char ch = sock_read(sock);
			map_clear_blacklist(map, x, y);
			
			if (ch == '@')
				map_set_tiletype(map, x, y, TT_HOMEBASE);
			else if (ch == '~')
				map_set_tiletype(map, x, y, TT_WATER);
			else if (ch == '#')
				map_set_tiletype(map, x, y, TT_WALL);
			else if (ch == '.')
				map_set_tiletype(map, x, y, TT_PLAIN);
			else 
				return NULL;
		}
		sock_read(sock);
	}
	return map;
}
int possible_moves(map_t *map, size_t x, size_t y, int ignore_robots)
{
	int result = 0, dir;
	for (dir = 0; dir < 4; dir++)
		if (move_is_possible(map, x, y, dir, ignore_robots)) {
			result <<= 8;
			result |= 1 << dir;
		}
	return result;
}
				
int move_is_possible(map_t *map, size_t x, size_t y, int dir, 
	int ignore_robots)
{
	listiterator_t *it;

	x += directions[dir][0];
	y += directions[dir][1];
	if (x < 1 || y < 1 ||  x > map->width || y > map->height)
		return 0;

	if (!ignore_robots) {
		for (it = robot_iterator(); it->current;
			it = listiterator_next(it)) {
			robot_t *robot = listiterator_data(robot_t *, it);
			if (robot->x == x && robot->y == y)
				return 0;
		}
		listiterator_free(it);
	}

	return (map_get_tiletype(map, x, y) != TT_WATER && 
		map_get_tiletype(map, x, y) != TT_WALL);
}
