#ifndef _MAP_H_INCLUDED
#define _MAP_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#include <assert.h>
#include <stdint.h>
#include <stdarg.h>
#include <stdint.h>
#include <sys/types.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>

#include "net.h"
#include "robot.h"

extern robot_t *myrobot;

typedef struct {
	uint8_t type;
	/* store the best achieved minimum distance to goal, there's
	 * no use to add the longer paths to search queue.
	 */
	int minimum_distance;
	uint8_t visited;
	int price;
	int blacklist;
} tile_t;

typedef struct {
	tile_t *data;
	size_t width;
	size_t height;
} map_t;

typedef struct {
	int x;
	int y;
} location_t;

enum {
	TT_HOMEBASE	= 0x01,
	TT_WATER	= 0x02,
	TT_WALL		= 0x04,
	TT_PLAIN	= 0x08
};

static inline int distance(int sx, int sy, int dx, int dy)
{
	return abs(sx - dx) + abs(sy - dy);
}

int possible_moves(map_t *map, size_t x, size_t y, int ignore_robots);
int move_is_possible(map_t *map, size_t x, size_t y, int dir,
	int ignore_robots);

static inline uint8_t map_get_tiletype(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	return map->data[map->width * y + x].type;
}

static inline void map_set_mindistance(map_t *map, size_t x, size_t y,
	unsigned int distance)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].minimum_distance = distance;
}

static inline unsigned int map_get_mindistance(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	return map->data[map->width * y + x].minimum_distance;
}

static inline void map_increase_price(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].price++;
}

static inline void map_clear_price(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].price = 0;
}

static inline int map_get_price(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	return map->data[map->width * y + x].price;
}

static inline void map_increase_visited(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].visited++;
}

static inline void map_clear_visited(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].visited = 0;
}

static inline int map_get_visited(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	return map->data[map->width * y + x].visited;
}

static inline void map_increase_blacklist(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].blacklist += 2;
}

static inline void map_decrease_blacklist(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	if (map->data[map->width * y + x].blacklist > 0)
		map->data[map->width * y + x].blacklist--;
}


static inline void map_clear_blacklist(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	map->data[map->width * y + x].blacklist = 0;
}

static inline int map_get_blacklist(map_t *map, size_t x, size_t y)
{
	x--;
	y--;
	assert(x < map->width && y < map->height);
	return map->data[map->width * y + x].blacklist;
}

void map_set_tiletype(map_t *map, size_t x, size_t y, uint8_t value);
map_t *map_read(sockbuf_t *sock, int width, int height);


void compute_reachable_locations(map_t *map, size_t x, size_t y);
void random_reachable_location(size_t *x, size_t *y);

static inline void logger(const char *fmt, ...)
{
	va_list args;
	FILE *fp;
	struct tm *tm;
	time_t t;
/*	fp = fopen("log", "a");
	va_start(args, fmt);
	time(&t);
	tm = localtime(&t);
	if (myrobot)
		fprintf(fp, "%d: %02d:%02d:%02d: ", myrobot->id, tm->tm_hour, tm->tm_min, tm->tm_sec);
	else
		fprintf(fp, "UNKNOWN: %02d:%02d:%02d: ", tm->tm_hour, tm->tm_min, tm->tm_sec);
	vfprintf(fp, fmt, args);
	va_end(args);
	fclose(fp);*/
}

#ifdef __cplusplus
};
#endif

#endif
