#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "net.h"

static sockbuf_t *sock_new(int sockfd);

sockbuf_t *init_network(char *host, int port)
{
	struct sockaddr_in sockaddr;
	struct hostent *hostent;
	struct protoent *proto;
	char buffer[32];
	int sockfd;
	hostent = gethostbyname(host);
	snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d", 
		hostent->h_addr_list[0][0],
		hostent->h_addr_list[0][1],
		hostent->h_addr_list[0][2],
		hostent->h_addr_list[0][3]);
	if (!inet_aton(buffer, &sockaddr.sin_addr))
		return NULL;
	sockaddr.sin_port = htons(port);
	sockaddr.sin_family = PF_INET;
	if ((proto = getprotobyname("tcp")) == NULL)
		return NULL;
	if ((sockfd = socket(PF_INET, SOCK_STREAM, proto->p_proto)) == -1)
		return NULL;
	if (connect(sockfd, (struct sockaddr *) &sockaddr,
		sizeof(struct sockaddr_in)) == -1) 
		return NULL;
	return sock_new(sockfd);
}

static sockbuf_t *sock_new(int sockfd)
{
	sockbuf_t *sock;

	sock = (sockbuf_t *) malloc(sizeof(sockbuf_t));
	sock->sockfd = sockfd;
	sock->pending = 0;

	return sock;
}

int sock_read(sockbuf_t *sock)
{
	uint8_t retval;
	int i;
begin:
	if (sock->pending) {
		retval = sock->buffer[0];
		for (i = 0; i < sock->pending; i++)
			sock->buffer[i] = sock->buffer[i + 1];
		sock->pending--;
		return retval;
	} 
	if ((sock->pending = read(sock->sockfd, 
		sock->buffer, SOCKBUFLEN)) == -1)
		return -1;

	if (sock->pending == 0) 
		usleep(10);
	goto begin;
}

char *sock_read_string_dynamic(sockbuf_t *sock)
{
	char *ptr, *nptr;
	int ch;
	int size = 1;

	/* compability for older reallocs */
	if ((ptr = (char *) malloc(1)) == NULL)
		return NULL;
	while ((ch = sock_read(sock)) != -1) {
		if (ch == '\n') {
			ptr[size - 1] = '\0';
			return ptr;
		}
		size++;
		if ((nptr = (char *) realloc(ptr, size)) == NULL)
			break;
		ptr = nptr;
		ptr[size - 2] = ch;
	}
	/* if we get here, it's an error */
	free(ptr);
	return NULL;
}

int sock_read_string(sockbuf_t *sock, char *line, size_t maxsize)
{
	if (!maxsize)
		return 0;
	while (--maxsize) {
		int ch;
		if ((ch = sock_read(sock)) == -1)
			return -1;
		if (ch == '\n')
			break;
		*line++ = ch;
	}
	*line = 0;
	return 0;
}

int sock_read_number(sockbuf_t *sock)
{
	int result = 0, ch;
	while ((ch = sock_read(sock)) != -1) {
		if (!isdigit(ch))
			return result;
		result = result * 10 + ch - '0';
	}
	return -1;
}


