#include <stdio.h>
#include <stdlib.h>

#include "path.h"

static location_t *reachable = NULL;
static int num_reachable = 0;

static void add_reachable(map_t *map, size_t x, size_t y)
{
	num_reachable++;
	reachable = (location_t *) realloc(reachable,
		num_reachable * sizeof(location_t));
	reachable[num_reachable - 1].x = x;
	reachable[num_reachable - 1].y = y;
	map_increase_visited(map, x, y);
}

static void do_dirty_work(map_t *map, size_t x, size_t y)
{
	int i;
	size_t tx, ty;
	for (i = 0; i < 4; i++) {
		if (!move_is_possible(map, x, y, i, 1)) 
			continue;
		tx = x + directions[i][0];
		ty = y + directions[i][1];
		if (map_get_visited(map, tx, ty))
			continue;
		add_reachable(map, tx, ty);
		do_dirty_work(map, tx, ty);
	}
}

void compute_reachable_locations(map_t *map, size_t x, size_t y)
{
	size_t tx, ty;

	/* for compability with older libc's */
	reachable = (location_t *) malloc(sizeof(location_t));

	for (ty = 1; ty <= map->height; ty++)
		for (tx = 1; tx <= map->width; tx++)
			map_clear_visited(map, tx, ty);

	do_dirty_work(map, x, y);

	for (ty = 1; ty <= map->height; ty++)
		for (tx = 1; tx <= map->width; tx++)
			map_clear_visited(map, tx, ty);

	printf("%d reachable locations for my robot.\n", num_reachable);
}

void random_reachable_location(size_t *x, size_t *y)
{
	int n;

	n = rand() % num_reachable;
	*x = reachable[n].x;	
	*y = reachable[n].y;	
}
