#include "map.h"
#include "robot.h"

list_t *robots = NULL;

static int robot_find_by_id(void *a, void *b)
{
	robot_t *ra;
	ra = (robot_t *) a;
	return ra->id == (int) b;
}

int robot_init(void)
{
	if ((robots = list_new(robot_find_by_id)) == NULL)
		return -1;
	return 0;
}

void robot_add_package(robot_t *robot, int id)
{
	intarray_append(robot->packages, id);
	robot->load += package_weight(id);
}

void robot_del_package(robot_t *robot, int id)
{
	intarray_delete(robot->packages, id);
	robot->load -= package_weight(id);
}

void robot_move(robot_t *robot, map_t *map, int dx, int dy)
{
	int i;
	robot->x += dx;
	robot->y += dy;
	robot->dx += dx;
	robot->dy += dy;
	robot->moved = 1;
	for (i = 0; i < robot->packages->size; i++)
		package_move(robot->packages->data[i], robot->x, robot->y);	
	map_increase_visited(map, robot->x, robot->y);	
}

void robot_reset_alive_status(void)
{
	listiterator_t *it;
	robot_t *robot;
	for (it = robot_iterator(); it->current; it = listiterator_next(it)) {
		robot = listiterator_data(robot_t *, it);
		robot->alive = 0;
	}
	listiterator_free(it);
}

listiterator_t *robot_iterator(void) 
{
	return listiterator_new(robots);
}

robot_t *robot_new(int id)
{
	robot_t *robot;

	robot = (robot_t *) malloc(sizeof(robot_t));
	robot->id = id;
	robot->x = 0;
	robot->y = 0;
	robot->load = 0;
	robot->packages = intarray_new();
	robot->dx = 0;
	robot->dy = 0;
	robot->maxbid = 1;

	return robot;
}

robot_t *robot_find(int id)
{
	robot_t *robot;
	if ((robot = (robot_t *) list_find(robots, (void *) id)) != NULL)
		return robot;
	robot = robot_new(id);
	list_add(robots, robot);
	return robot;
}
void compute_price_mask(map_t *map, robot_t *myrobot)
{
	int x, y, tx, ty;
	listiterator_t *it;
	for (y = 1; y <= map->height; y++)
		for (x = 1; x <= map->width; x++)
			map_clear_price(map, x, y);
	for (it = robot_iterator(); it->current; it = listiterator_next(it)) {
		robot_t *robot = listiterator_data(robot_t *, it);
		if (robot == myrobot)
			continue;
		for (ty = -1; ty <= 1; ty++)
			for (tx = -1; tx <= 1; tx++) {
				x = tx + robot->x;
				y = ty + robot->y;
				if (x < 1 || y < 1 ||
					x > map->width || y > map->height)
					continue;
				map_increase_price(map, x, y);
			}
	}
	listiterator_free(it);
}


