#include <stdio.h>
#include <stdlib.h>

#include "util.h"

/* DELETE operation */  
void *list_car_delete(list_t *list)
{
	void *retval = NULL;
	listlink_t *next;

	if (list->list) {
		retval = list->list->data;
		next = list->list->next;
		free(list->list);
		list->list = next;
#ifdef PROFILE
		list->size--;
#endif
	}

	return retval;
}

void list_free(list_t *list)
{
	listlink_t *n, *p = list->list;
	while (p) {
		n = p->next;
		free(p);
		p = n;
	}
	free(list);
}

void *list_find(list_t *list, void *key)
{
	listlink_t *p;

	for (p = list->list; p; p = p->next)
		if (list->compare(p->data, key))
			return p->data;
	
	return NULL;
}

void *list_new(int (*compare)(void *, void *))
{
	list_t *list;

	list = (list_t *) malloc(sizeof(list_t));
	list->list =  NULL;
	list->compare = compare;
#ifdef PROFILE
	list->size = 0;
#endif

	return list;
}

/* INSERT operation */
void list_add(list_t *list, void *data)
{
	listlink_t *link;

	link = (listlink_t *) malloc(sizeof(listlink_t));
	link->data = data;
	link->next = list->list;
	list->list = link;
#ifdef PROFILE
	list->size++;
#endif 
}

/* INSERT operation */
void list_add_priority(list_t *list, void *data, int (*pfunc)(void *, void *))
{
	listlink_t *prev, *link;

	prev = NULL;
	for (link = list->list; link; link = link->next) {
		if (pfunc(data, link->data) <= 0)
			break;
		prev = link;
	}
	if (!prev) 
		list_add(list, data);
	else {
		listlink_t *nlink;

		nlink = (listlink_t *) malloc(sizeof(listlink_t));
		nlink->data = data;
		nlink->next = link;
		prev->next = nlink;
#ifdef PROFILE
		list->size++;
#endif
	}
}

void *xmalloc(size_t size)
{
	void *ptr = malloc(size);
	if (ptr == NULL)
		abort();
	return ptr;
}

void *xrealloc(void *ptr, size_t size)
{
	ptr = realloc(ptr, size);
	if (ptr == NULL)
		abort();
	return ptr;
}
